import ReadingProgress from "@components/ReadingProgress.svelte";
import { App, Modal } from "obsidian";
import { mount, unmount } from "svelte";

export class ReadingProgressModal extends Modal {
	private component: ReturnType<typeof ReadingProgress> | undefined;

	constructor(
		app: App,
		private readonly pageLength: number,
		private readonly onSubmit: (pageNumber: number) => void
	) {
		super(app);
	}

	onOpen(): void {
		this.component = mount(ReadingProgress, {
			target: this.contentEl,
			props: {
				pageLength: this.pageLength,
				onSubmit: (pageNumber) => {
					this.onSubmit(pageNumber);
					this.close();
				},
			},
		});
	}

	onClose(): void {
		if (this.component) {
			unmount(this.component);
			this.component = undefined;
		}
	}

	static createAndOpen(app: App, pageLength: number): Promise<number> {
		return new Promise((resolve) => {
			const modal = new ReadingProgressModal(
				app,
				pageLength,
				(pageNumber) => {
					resolve(pageNumber);
				}
			);
			modal.open();
		});
	}
}
